'' =================================================================================================
''
''   File....... jm_ironmac_v1f.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2014 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 20 SEP 2013
''
'' =================================================================================================

{

  This basic IronMac program is a light show controlled by PB1 (near P0 on left header). A quick
  press and release of PB1 changes the show state. To clear the display and put the badge in low-
  power mode, press and hold PB1 for about 2 seconds.

  To restart the badge, press and release PB1 and PB4 (near P16 on right header)

}


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq               ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us

  ' speed settings for power control/reduction
  ' -- use with clkset() instruction

  XT1_P16  = %0_1_1_01_111                                      ' 16x crystal (5MHz) = 80MHz
  XT1_PL8  = %0_1_1_01_110 
  XT1_PL4  = %0_1_1_01_101                                      
  XT1_PL2  = %0_1_1_01_100
  XT1_PL1  = %0_1_1_01_011
  RC_SLOW  = %0_0_0_00_001                                      ' 20kHz


con { io pins }

  RX1    = 31                                                   ' programming / terminal
  TX1    = 30
  
  SDA    = 29                                                   ' eeprom / i2c
  SCL    = 28

  PAD3   = 27                                                   ' touch pads
  PAD2   = 26
  PAD1   = 25
  PAD0   = 24
  
  LED7   = 23                                                   ' leds
  LED6   = 22
  LED5   = 21
  LED4   = 20
  LED3   = 19
  LED2   = 18
  LED1   = 17
  LED0   = 16

' IR_IN  = 15                                                   ' badge ir input
' IR_OUT = 14                                                   ' badge ir output


  ' IronMac Shield IO

  PB4    =  6
  PB3    =  5
  PB2    =  4
  PB1    =  3

  IR_IN  =  2
  IR_OUT =  1
  
  WSLEDS =  0


con { io configuration }

  IS_OFF    =  0                                                ' all bits off   
  IS_ON     = -1                                                ' all bits on    

  IS_LOW    =  0                                                
  IS_HIGH   = -1                                                

  IS_INPUT  =  0
  IS_OUTPUT = -1


con

  ACTIVE_STATES = 9
  

con { pst formatting }

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR
  #14, GOTOX, GOTOY, CLS


obj

  term : "fullduplexserial64"

  leds : "jm_ws2812"                                            ' ws2812 led driver (auto refresh)
  irtx : "jm_sircs_tx"                                          ' SIRCS output
  irrx : "jm_sircs_rx"                                          ' SIRCS input
  prng : "jm_prng"                                              ' pseudo-random numbers
  ee   : "jm_24xx512"                                           ' eeprom access
  tmr1 : "jm_eztimer"                                           ' asynchronous timer
    

dat

  state       long      1 


pub main | check

  setup

  check := state

  repeat
    leds.off                                                    ' clear for next animation

    if (check <> state)                                         ' send Goon/Uber command msg       
      irtx.tx($DC22, 16, 3)    
      check := state           
  
    case state                                                  ' run new animation
      0 : shut_down                    
      1 : color_round(@RWB, 3, 50)   
      2 : morph
      3 : rainbow_spin                   
      4 : spinner                    
      5 : arc_cylon
      6 : heart_beat                     
      7 : random_fill 
      8 : rainbow_cascade
      9 : skyward            

      
pub setup

'' Setup IO and objects for application

' term.start(RX1, TX1, %0000, 115_200)                          ' terminal io for testing

  leds.start_b(WSLEDS, 10)                                      ' uses WS2812B LEDs
  irtx.start(IR_OUT, 36_000)                       
' irrx.start(IR_IN)
  button_start(PB1)                                             ' background button monitor
      
  prng.seed(-cnt, -cnt ~> 4, $dead_beef, cnt << 4, cnt)
  ee.start(%000)                                                ' boot eeprom 


pub shut_down | ch, check

' Kill active cogs and enter low-power mode

  leds.off
  pause(10)
  
  leds.stop
  irtx.stop
  irrx.stop
  button_stop

  clkset(RC_SLOW, 20_000)                                       ' slow down!
  
  ' press and release PB1 and PB4 to reboot  
  
  repeat
    waitcnt(cnt + 2000)
  until (ina[PB4..PB1] == %1001)

  repeat
    waitcnt(cnt + 2000)
  until (ina[PB4..PB1] == %0000)

  reboot
    

dat

  RWB         long      $18_00_00, $10_10_10, $00_00_20

  RGB         long      $18_00_00, $00_20_00, $00_00_20

  Rainbow     long      $17_00_00, $14_0A_00, $0C_0C_00, $00_20_00, $00_00_20, $0A_00_0A, $08_08_08


pub color_round(p_map, count, ms) | mystate, p_color, color, ch 

'' Change colors in rotating pattern
'' -- p_map is pointer to color map array
'' -- count is number of colors to display
'' -- ms is delay per pixel

  mystate := state                                              ' what state is calling me? 

  repeat
    p_color := p_map                                            ' get base of map
    repeat count                                                ' for each color
      color := long[p_color]                                    ' read it
      p_color += 4                                              ' point to next
      repeat ch from 0 to 9                                     ' loop through pixels
        leds.set(ch, $00_00_00)                                 ' clear it
        pause(ms)                                               ' wait
        leds.set(ch, color)                                     ' paint new color
        if (state <> mystate)   
          return                
        
     
pub morph | mystate, idx

'' Smooth morph of color to color

  mystate := state

  repeat
    repeat idx from 0 to 255
      leds.set_all(leds.wheelx(idx, $20))
      pause(8)
      if (state <> mystate)
        return


pub rainbow_spin | mystate, ch, phase, bump

'' Displays rotating rainbow effect

  mystate := state
  
  repeat
    repeat ch from 0 to 9
      phase := ch * (255 / 10)  
      leds.set(ch, leds.wheelx((phase+bump) & $FF, $20))
    pause(5)
    --bump
    bump &= $FF

    if (mystate <> state)
      return  
  

pub spinner | mystate, idx, ch

'' Bi-color spinning display                            

  mystate := state

  repeat
    repeat idx from 9 to 0
      ch := idx
      repeat 5
        leds.set(ch, $00_20_00)
        ch := (ch + 1) // 10
      repeat 5
        leds.set(ch, $00_00_20)
        ch := (ch + 1) // 10      
      pause(100)
      if (state <> mystate)
        return


pub arc_cylon | mystate, cidx, color, idx, ch

'' Arc shaped Larsen scanner

  mystate := state                                            

  repeat
    repeat cidx from 0 to 2
      color := lookupz(cidx: $30_00_00, $00_30_00, $00_00_30)
      repeat idx from 0 to 7
        ch := lookupz(idx : 6,7,8,9,0,1,2,3)
        leds.off
        leds.set(ch, color)
        pause(500/7)
        if (state <> mystate)
          return
      repeat idx from 0 to 7
        ch := lookupz(idx : 4,3,2,1,0,9,8,7) 
        leds.off           
        leds.set(ch, color)
        pause(500/7)
        if (state <> mystate)
          return


pub heart_beat | mystate, level, color

   mystate := state

   repeat
     repeat level from 0 to $30
       color := level << 16
       leds.set_all(color)
       pause(2)
       
     repeat level from $30 to $01
       color := level << 16
       leds.set_all(color)
       pause(3)     

     pause(50)

     repeat level from 0 to $30
       color := level << 16
       leds.set_all(color)
       pause(2)
       
     repeat level from $30 to $04
       color := level << 16     
       leds.set_all(color)
       pause(20)  
       if (state <> mystate)
         return


pub random_fill | mystate, last, phase, color, lit, ch, mask

'' Fills LED ring randomly; red, then green, then blue

  mystate := state

  last := -1

  repeat
    repeat phase from 0 to 2
      color := RGB[phase]
      lit := 0
      repeat until (lit == $3FF)
        repeat
          ch := (prng.random >> 1) // 10
          mask := 1 << ch
        until ((ch <> last) and ((mask & lit) == 0))
        leds.set(ch, color)
        pause(50)  
        lit |= mask
        last := ch                                    
        if (state <> mystate)
          return
        
        
dat
  
  HPairs      byte      -1,  0
              byte       9,  1
              byte       8,  2
              byte       7,  3
              byte       6,  4
              byte       5, -1

  VPairs      byte       8,  7
              byte       9,  6
              byte       0,  5
              byte       1,  4
              byte       2,  3
    
      
pub rainbow_cascade | mystate, hp, phase, ch, bump
  
  mystate := state

  repeat
    repeat hp from 0 to 5                                       ' loop through horizontal pairs
      phase := hp * (255 / 6)                                   ' calc color phase for this pair
      ch := HPairs[hp*2+0]                                      ' get led ch
      if ((ch => 0) and (ch =< 9))                              ' if valid
        leds.set(ch, leds.wheelx((phase+bump) & $FF, $20))      ' set the color
      ch := HPairs[hp*2+1]   
      if ((ch => 0) and (ch =< 9))
        leds.set(ch, leds.wheelx((phase+bump) & $FF, $20))            
    pause(1)
    --bump                                                      ' decrement phase bump
    bump &= $FF                                                 ' keep 0..255 
    
    if (mystate <> state) 
      leds.off
      return   


pub skyward | mystate, hp, phase, ch, timing

  mystate := state

  timing := 100

  repeat
    repeat hp from 5 to 0    
      ch := HPairs[hp*2+0]
      if ((ch => 0) and (ch =< 9)) 
        leds.set(ch, $10_00_10)
      ch := HPairs[hp*2+1]           
      if ((ch => 0) and (ch =< 9))   
        leds.set(ch, $10_00_10)         
      pause(timing)
        
      if (mystate <> state) 
        return     

    repeat hp from 5 to 0
      ch := HPairs[hp*2+0]
      if ((ch => 0) and (ch =< 9)) 
        leds.set(ch, $00_00_00)
      ch := HPairs[hp*2+1]           
      if ((ch => 0) and (ch =< 9))   
        leds.set(ch, $00_00_00)         
      pause(timing)
              
      if (mystate <> state) 
        return

    timing := timing * 8 / 10                                   ' accellerate
    if (timing < 25)                                            ' fast enough?
      timing := 100                                             ' yes, reset
      
      repeat phase from $80 to $00                              ' flash and fade
        leds.set_all(phase << 16 | phase)
        pause(1000 / 128)
        if (mystate <> state) 
          return              

  
con

  { --------------------------------------------------- }
  {  B A C K G R O U N D   B U T T O N   M O N I T O R  }
  { --------------------------------------------------- }
  

var

  long  btncog
  long  btnstack[32]
  

pri button_start(pin)

'' Starts background button monitor

  button_stop 

  btncog := cognew(button_cog(pin), @btnstack) + 1


pri button_stop

'' Stops background button monitor

  if (btncog)
    cogstop(btncog-1)
    btncog := 0

  
pri button_cog(pin) | scan, t, btimer

  dira[pin] := 0                                                ' make input

  scan := %00                                                   ' clear scan
  btimer := 0                                                   ' clear timer

  t := cnt                                                      ' sync timer
  repeat
    waitcnt(t += MS_001)                                        ' wait 1ms
    scan := ((scan << 1) & %11) | ina[pin]                      ' update scan
    case scan
      %00:                                                      ' no press
        btimer := 0

      %01:                                                      ' new press
        btimer := 0

      %11:                                                      ' button held
        btimer += 1

      %10:                                                      ' button release
        if ((btimer => 50) and (btimer =< 500))
          if (++state > ACTIVE_STATES)
            state := 1
          ee.wr_long(@state, state)                             ' save state to ee
          t := cnt                                              ' re-sync loop timer
                                       
        elseif ((btimer => 1000) and (btimer =< 3000))          ' if long press                  
          if (state <> 0)                                       '  if not off
            state := 0                                          '   kill it


con

  { ----------------------------- }
  {  B A D G E   F E A T U R E S  }
  { ----------------------------- }


pub set_leds(pattern)

'' Sets LED pins to output and writes pattern to them
'' -- swaps LSB/MSB for correct binary output

  outa[LED0..LED7] := pattern                                   ' write pattern to LEDs
  dira[LED0..LED7] := IS_HIGH                                   ' make LED pins outputs

  
pub read_pads

'' Reads and returns state of touch pad inputs
'' -- swaps LSB/MSB for correct binary input

  outa[PAD3..PAD0] := IS_HIGH                                   ' charge pads (all output high)   
  dira[PAD3..PAD0] := IS_OUTPUT
    
  dira[PAD3..PAD0] := IS_INPUT                                  ' float pads   
  pause(50)                                                     ' -- allow touch to discharge

  return (!ina[PAD3..PAD0] & $0F) >< 4                          ' return "1" for touched pads


con

  { --------------- }
  {  L E D   F U N  }
  { --------------- }


var

  long  anicog                                                  ' cog running animation
  long  anistack[32]                                            ' stack space for Spin cog
  

pri start_animation(p_table, cycles)

'' Start animation in background cog
'' -- allows LED animation while doing other processes
'' -- p_table is pointer (address of) animation table
'' -- set cycles to 0 to run without stopping

  stop_animation
  
  anicog := cognew(run_animation(p_table, cycles), @anistack) + 1  

  return anicog                                                 ' return cog used


pri stop_animation

'' Stop animation if currently running

  if (anicog)                                                   ' if running
    cogstop(anicog - 1)                                         ' stop the cog
    anicog := 0                                                 ' mark stopped 


pri run_animation(p_table, cycles) | p_leds

'' Run animation
'' -- p_table is pointer (address of) animation table
'' -- cycles is number of iterations to run
''    * 0 cycles runs "forever"
'' -- usually called with start_animation()

  if (cycles =< 0)
    cycles := POSX                                              ' run "forever"

  repeat cycles
    p_leds := p_table                                           ' point to table
    repeat byte[p_leds++]                                       ' repeat for steps in table
      set_leds(byte[p_leds++])                                  ' update leds
      pause(byte[p_leds++])                                     ' hold
      
  anicog := 0                                                   ' mark stopped
  cogstop(cogid)                                                ' stop this cog
    

dat

  ' Animation tables for LEDs
  ' -- 1st byte is number of steps in animation sequence
  ' -- each step holds pattern and hold time (ms)
  ' -- for delays > 255, duplicate pattern + delay

  Cylon       byte      (@Cylon_X - @Cylon) / 2 + 1 
              byte      %10000000, 125
              byte      %01000000, 125
              byte      %00100000, 125 
              byte      %00010000, 125
              byte      %00001000, 125  
              byte      %00000100, 125  
              byte      %00000010, 125  
              byte      %00000001, 125
              byte      %00000010, 125
              byte      %00000100, 125
              byte      %00001000, 125  
              byte      %00010000, 125  
              byte      %00100000, 125    
  Cylon_X     byte      %01000000, 125

                    
  Chaser      byte      (@Chaser_X - @Chaser) / 2 + 1      
              byte      %10010010,  75
              byte      %00100100,  75
  Chaser_X    byte      %01001001,  75

                     
  InOut       byte      (@InOut_X - @InOut) / 2 + 1  
              byte      %10000001, 100
              byte      %01000010, 100
              byte      %00100100, 100
              byte      %00011000, 100
              byte      %00100100, 100
  InOut_X     byte      %01000010, 100

          
  Police      byte      (@Police_X - @Police) / 2 + 1 
              byte      %11001100,  75
              byte      %11110000,  75
              byte      %11001100,  75
              byte      %11110000,  75         
              byte      %00001111,  75
              byte      %00110011,  75
              byte      %00001111,  75
  Police_X    byte      %00110011,  75

  
  Blasted     byte      (@Blasted_X - @Blasted) / 2 + 1
              byte      %00000000,  50 
              byte      %00011000,  50
              byte      %00111100,  50
              byte      %01111110,  50
              byte      %11111111,  50
              byte      %00000000,  50 
              byte      %00011000,  50
              byte      %00111100,  50
              byte      %01111110,  50
              byte      %11111111,  50
              byte      %00000000,  50 
              byte      %00011000,  50
              byte      %00111100,  50
              byte      %01111110,  50
              byte      %11111111,  50
              byte      %00000000,  50    
              byte      %11111111,  50    
              byte      %00000000,  50
              byte      %11111111,  50
              byte      %00000000,  50    
              byte      %11111111,  50    
              byte      %00000000,  50
              byte      %11111111,  50
              byte      %00000000,  50    
              byte      %11111111,  50    
  Blasted_X   byte      %00000000,  50
          
  
con   

  { ------------- }
  {  B A S I C S  }
  { ------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
      

pub high(pin)

'' Makes pin output and high

  outa[pin] := IS_HIGH
  dira[pin] := IS_OUTPUT


pub low(pin)

'' Makes pin output and low

  outa[pin] := IS_LOW
  dira[pin] := IS_OUTPUT


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := IS_OUTPUT


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := IS_INPUT

  return ina[pin]

  
con

  { ---------------------------- }
  {  S H I E L D  T E S T I N G  }
  { ---------------------------- }
  

dat

  Menu        byte      "---------------------", CR
              byte      " IronMac Shield Test ", CR   
              byte      "---------------------", CR
              byte      CR
              byte      "[1] LEDs", CR
              byte      "[2] Buttons", CR
              byte      "[3] IR", CR
              byte      CR
              byte      0


pub shield_test | c

  ' open terminal and press a key

  term.rxflush
  term.rx

  repeat
    term.tx(CLS)
    term.str(@Menu)
    c := term.rx
    case c
      "1" : test_leds
      "2" : test_buttons
      "3" : test_ir
           

pub test_leds | ch, c

  term.rxflush

  repeat
    leds.set_all($08_00_00)
    pause(1000)
    leds.set_all($00_08_00) 
    pause(1000)
    leds.set_all($00_00_08) 
    pause(1000)
    c := term.rxcheck
    if (c == CR)
      leds.off
      return

      
pub test_buttons | c

  term.tx(CLS)
  term.rxflush

  repeat
    term.tx(HOME)
    term.bin(ina[PB4..PB1], 4)
    c := term.rxtime(50)
    if (c == CR)
      return
  

dat

  IR_Test0              word    $A5A
  IR_Test1              word    $5A5
  IR_Test2              word    $00F
  IR_Test3              word    $0F0
  IR_Test4              word    $F00
  
    
pub test_ir | checks, idx

  term.tx(CLS)
  term.str(string("IR Test", CR))

  checks := 0

  repeat idx from 0 to 4
    repeat 3
      irrx.enable
      irtx.tx(IR_Test0[idx], 12, 1)
      repeat while (irtx.busy)                                  ' hold for frame transmission
      if (irrx.bit_count == 12)                                 ' good bit count?
        if (irrx.rxcheck == IR_Test0[idx])                      ' good code?
          checks += 1
          quit
          
  if (checks == 5)
    term.str(string("-- Pass!"))
  else
    term.str(string("-- Fail!"))

  pause(2000)


dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  